FishingAce = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceHook-2.1", "AceDB-2.0", "Metrognome-2.0")

local L = AceLibrary("AceLocale-2.2"):new("FishingAce")
local metro = AceLibrary("Metrognome-2.0")

local options = { 
    type='group',
    args = {
         auto = {
            type = 'toggle',
            name = L["Auto Loot"],
            desc = L["If set, Fishing Ace will turn Auto Loot on while you're fishing."],
            get = "IsAutoLoot",
            set = "ToggleAutoLoot",
        },
        sound = {
            type = 'toggle',
            name = L["Enhance Sounds"],
            desc = L["If set, Fishing Ace enhance the ambient sound while you're fishing."],
            get = "IsEnhanceSounds",
            set = "ToggleEnhanceSounds",
        },
    }
}

FishingAce:RegisterDB("FishingBuddyDBPC")
FishingAce:RegisterDefaults("profile", {
    autoLoot = true,
    enhanceSounds = true,
} )
FishingAce:RegisterChatCommand(L["Slash-Commands"], options)

local overrideOn = false
FishingAceButton = nil
local function PostCastUpdate(self)
   if ( not InCombatLockdown() ) then
      metro:StopMetro("FishingAce")
      FishingAceButton:Hide()
      if ( overrideOn ) then
         ClearOverrideBindings(FishingAceButton)
         overrideOn = false
      end
   end
end

local function HideAwayAll(self, button, down)
   if ( overrideOn ) then
      metro:StartMetro("FishingAce")
   end
end

function FishingAce:OnInitialize()
   local btn = CreateFrame("CheckButton", "CastFishingButton", UIParent, "ActionBarButtonTemplate");
   btn:SetScript("OnAttributeChanged", ActionButton_Update);
   if ShiftClickFix_ActionButton then
      ShiftClickFix_ActionButton(btn);
   end
   btn:SetPoint("LEFT", UIParent, "RIGHT", 10000, 0);
   btn:SetFrameStrata("LOW");
   btn:EnableMouse(true);
   btn:SetWidth(100);
   btn:SetHeight(100);
   btn:RegisterForClicks("RightButtonUp");
   btn:Hide();
   btn:SetScript("PostClick", HideAwayAll);
   FishingAceButton = btn;
end

-- Something else that should be in a library
local function SplitLink(link)
   if ( link ) then
      local _,_, color, item, name = string.find(link, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:%-?%d+)|h%[(.-)%]|h|r");
      return color, item, name;
   end
end

local function IsFishingPole()
   -- Get the main hand item texture
   local slot = GetInventorySlotInfo("MainHandSlot");
   local itemTexture = GetInventoryItemTexture("player", slot);
   -- If there is infact an item in the main hand, and it's texture
   -- that matches the fishing pole texture, then we have a fishing pole
   if ( itemTexture and string.find(itemTexture, "INV_Fishingpole") ) then
      local link = GetInventoryItemLink("player", slot);
      local _, id, _ = SplitLink(link);
      -- Make sure it's not "Nat Pagle's Fish Terminator"
      if ( id and not string.find(id, "^19944") ) then
         return true;
       end
   end
   return false;
end

local function HijackCheck()
   if ( not InCombatLockdown() and IsFishingPole() ) then
      return true;
   end
end

local function FindSpellID(thisone)
   local id = 1;
   local spellTexture = GetSpellTexture(id, BOOKTYPE_SPELL);
   while (spellTexture) do
      if (spellTexture and spellTexture == thisone) then
         return id;
      end
      id = id + 1;
      spellTexture = GetSpellTexture(id, BOOKTYPE_SPELL);
   end
   return nil;
end

local FISHINGTEXTURE = "Interface\\Icons\\Trade_Fishing"
local FishingSpellID, FishingSpellName

local function GetFishingSpellID()
   if ( not FishingSpellID or not FishingSkillName) then
      FishingSpellID = FindSpellID(FISHINGTEXTURE);
   end
   if ( FishingSpellID and not FishingSkillName ) then
      FishingSkillName = GetSpellName(FishingSpellID, BOOKTYPE_SPELL);
   end
   return FishingSpellID, FishingSkillName;
end

-- do everything we think is necessary when we start fishing
-- even if we didn't do the switch to a fishing pole
local efsv = nil;
local function EnhanceFishingSounds(self, enhance)
   if ( self.db.profile.enhanceSounds ) then
      if ( enhance ) then
         local mv = tonumber(GetCVar("MusicVolume"));
         local av = tonumber(GetCVar("AmbienceVolume"));
         local sv = tonumber(GetCVar("SoundVolume"));
         if ( not efsv ) then
         -- collect the current value
            efsv = {};
            efsv["MusicVolume"] = mv;
            efsv["AmbienceVolume"] = av;
            efsv["SoundVolume"] = sv;
         -- turn 'em off!
            SetCVar("SoundVolume", 1.0);
            SetCVar("MusicVolume", 0.0);
            SetCVar("AmbienceVolume", 0.0);
         end
      else
         if ( efsv ) then
            for setting, value in pairs(efsv) do
               SetCVar(setting, value);
            end
            efsv = nil;
         end
      end
   end
end

local function StartFishingMode(self)
   if ( not self.startedFishing ) then
      -- Disable Click-to-Move if we're fishing
      if ( GetCVar("autointeract") == "1" ) then
         self.resetClickToMove = true;
         SetCVar("autointeract", "0");
      end
      if ( self.db.profile.autoLoot ) then
         if ( not GetAutoLootDefault() ) then
            self.resetAutoLoot = true;
            SetAutoLootDefault(true);
         end
      end
      self.startedFishing = GetTime();
      EnhanceFishingSounds(self, true);
   end
end

local function StopFishingMode(self)
   if ( self.startedFishing ) then
      EnhanceFishingSounds(self, false);
      self.startedFishing = nil;
   end
   if ( self.resetClickToMove ) then
      -- Re-enable Click-to-Move if we changed it
      SetCVar("autointeract", "1");
      self.resetClickToMove = nil;
   end
   if ( self.resetAutoLoot ) then
      SetAutoLootDefault(false);
      self.resetAutoLoot = nil;
   end
end

local function FishingMode(self)
   if ( IsFishingPole() ) then
      StartFishingMode(self);
   else
      StopFishingMode(self);
   end
end

local ACTIONDOUBLEWAIT = 0.4;
local lastClickTime = nil;
local function CheckForDoubleClick()
   if ( lastClickTime ) then
      local pressTime = GetTime();
      local doubleTime = pressTime - lastClickTime;
      lastClickTime = pressTime;
      if ( doubleTime < ACTIONDOUBLEWAIT ) then
         return true;
      end
   end
   lastClickTime = GetTime();
   return false;
end

-- handle mouse up and mouse down in the WorldFrame so that we can steal
-- the hardware events to implement 'Easy Cast'
-- Thanks to the Cosmos team for figuring this one out -- I didn't realize
-- that the mouse handler in the WorldFrame got everything first!
local function WF_OnMouseDown(...)
   -- Only steal 'right clicks' (self is arg #1!)
   local button = select(2, ...);
   if ( button == "RightButton" and HijackCheck() ) then
      if ( CheckForDoubleClick() ) then
          -- We're stealing the mouse-up event, make sure we exit MouseLook
         if ( IsMouselooking() ) then
            MouselookStop();
         end
         local id,name = GetFishingSpellID();
         if ( name ) then
            FishingAceButton:SetAttribute("type", "spell");
            FishingAceButton:SetAttribute("spell", name);
            SetOverrideBindingClick(FishingAceButton, true, "BUTTON2", "FishingAceButton");
            overrideOn = true;
         end
      end
   end
end

function FishingAce:OnEnable()
   self:RegisterMetro("FishingAce", PostCastUpdate, 1, self)
   self:HookScript(WorldFrame, "OnMouseDown", WF_OnMouseDown);
   self:RegisterEvent("PLAYER_ENTERING_WORLD");
   self:RegisterEvent("PLAYER_LEAVING_WORLD");
end

function FishingAce:OnDisable()
--   self:UnRegister("FishingAce")
--   AceHook:Unhook(WorldFrame, "OnMouseDown");
--   self:UnregisterEvent("SPELLS_CHANGED");
end

function FishingAce:IsEnhanceSounds()
   return self.db.profile.enhanceSounds
end

function FishingAce:ToggleEnhanceSounds()
   EnhanceFishingSounds(self, false)
   self.db.profile.autoLoot = not self.db.profile.enhanceSounds
end

function FishingAce:IsAutoLoot()
   return self.db.profile.autoLoot
end

function FishingAce:ToggleAutoLoot()
    self.db.profile.autoLoot = not self.db.profile.autoLoot
end

function FishingAce:SPELLS_CHANGED()
   -- Fishing might have moved, go look again
   FishingSpellID = nil
end

function FishingAce:ITEM_LOCK_CHANGED()
   FishingMode(self)
end

function FishingAce:PLAYER_ENTERING_WORLD()
   self:RegisterEvent("ITEM_LOCK_CHANGED");
   self:RegisterEvent("SPELLS_CHANGED");
end

function FishingAce:PLAYER_LEAVING_WORLD()
   self:UnregisterEvent("ITEM_LOCK_CHANGED");
   self:UnregisterEvent("SPELLS_CHANGED");
end
